/**================================================================================================
Hochschule Luzern - Technik & Architektur                                  Mikrocontroller - HS2012
Copyright 2012 Felix Rohrer                                                    http://hslu.ximit.ch
---------------------------------------------------------------------------------------------------
Name        : main.c
Author      : Felix Rohrer <felix.rohrer@stud.hslu.ch>
Version     : 2012-10-07v1.0
Description : Taschenrechner polnische Notation
=================================================================================================*/

#include <stdio.h>
#include <stdlib.h>		/* fuer atof() */
#include <math.h>		/* fuer sin() und co */
#include "calc.h"		/* Eigene Methoden einbinden */

/* Konstanten */
#define MAXOP 100		/* max Laenge von Operand und/oder Operator */
#define PI 3.14159265	/* PI */

/**
 * The main procedure to run the application.
 * @param argc Number of command line arguments
 * @param argv The forwarded command line arguments
 * @return Application return (error) code
 */
int main(int argc, char** argv)
{
	/* Taschenrechner in umgekehrter polnischer Notation - Beispiel S. 74 K&R */
	
	int type;
	double op2;
	char s[MAXOP];

	//Oeffnet das File "inputfile.txt" in dem Pfad von main.c und nimmt das File als
	//Standard Eingabe anstelle Tastatur
	freopen("inputfile1.txt", "r", stdin);
	while ((type = getop(s)) != EOF) {
		switch (type) {
		case NUMBER:	// Zahl auf Stack pushen
			push(atof(s));
			break;
		case '+':		// + Plus
			push(pop() + pop());
			break;
		case '-':		// - Minus
			op2 = pop();
			push(pop() - op2);
			break;
		case '*':		// * Multiplikation
			push(pop() * pop());
			break;
		case '/':		// / Division
			op2 = pop();
			if (op2 != 0.0) {
				push(pop() / op2);
			} else {
				printf("Error: Division durch 0!\n");
			}
			break;
		case FN_SIN:	// Sinus
			push(sin(pop() * PI / 180.0));
			break;
		case FN_COS:	// Cosinus
			push(cos(pop() * PI / 180.0));
			break;
		case FN_TAN:	// Tangens
			push(tan(pop() * PI / 180.0));
			break;
		case FN_EXP:	// Exp, e^x
			push(exp(pop()));
			break;
		case FN_POWER:	// pow, x^y
			op2 = pop();
			push(pow(pop(), op2));
			break;
		case '\n':		// Resultat Ausgeben
			printf("Result:\t%.8g\n", pop());
			break;
		default:		// Unbekannter Befehl
			printf("Error: Unbekannter Befehl: %s\n", s);
			break;
		} // switch (type)
	} // while ( != EOF)
	
	fclose(stdin); // Schliesst die Standard-Eingabe stdin (das File von freopen)

	return(EXIT_SUCCESS);
}
